'SE_NavCenter.csc
'Copyright 2003 by Abe Hendin <venturadev@yourspeed.com>. All rights reserved.
'Version 1.03, May 4, 2003

'This script allows you to jump to SUBs and FUNCTIONs with a double-click.
'It also displays SUB and FUNCTION parameters, and allows you to bookmark a 
'single line for later return. I've got the script assigned to ALT-N for ease
'of access.
'
'
'NOTES AND KNOWN ISSUES:
'All shortcuts except those defined for the script are disabled while the
'script is running. This is a limitation of CorelScript.
'
'The dialog position will not be saved or updated unless you click the 
'Reload button.
'
'When creating or returning to a bookmark,
'the script looks for a line that contains just the script filename 
'*with extension*, as it appears in the first line of this script, within 
'the first 5 lines of the script. It is able to strip the following comment 
'formats: "REM ", "' ", "'", and trailing spaces.
'Unpredictable results may occur with other formats.
'
'A "bini" file is stored in the Graphics8\CONFIG folder, as returned from the registry.
'I made the init file extension "bini" to make it easy to delete old bookmark 
'files if/when you decide to clean out your CONFIG folder.
'
'Version 1.03 corrects a Script Editor navigation bug whereby .GotoLine fails 
'if text is selected.

DECLARE SUB NavCenterDlgHandler(BYVAL ControlID%, BYVAL Event%)
DECLARE SUB LoadPosition
DECLARE SUB SavePosition
DECLARE SUB LoadSubsAndFunctions
DECLARE SUB BookmarkLine
DECLARE SUB GotoBookmark
DECLARE SUB ExtractFilename
DECLARE SUB CreateNewFilenameLine
DECLARE SUB CreateIniFile

#include "ScpConst.csi"
GLOBAL i AS INTEGER, iretval AS INTEGER
GLOBAL CurrLineNum AS LONG, sLoseREM AS STRING, sFilename AS STRING
GLOBAL bCancel AS BOOLEAN
GLOBAL DlgLeft AS INTEGER, DlgTop AS INTEGER

GLOBAL sEntryArr(1) AS STRING, sEntryLineNumArr(1) AS STRING, sParamsArr(1) AS STRING
GLOBAL iArrItems AS INTEGER

'DlgLeft, DlgTop, 
BEGIN DIALOG OBJECT NavCenterDlg DlgLeft, DlgTop, 208, 242, "Script Navigation Center", SUB NavCenterDlgHandler
	TEXT  7, 6, 192, 8, .SF_ListBoxLbl, "&SUB and FUNCTION list (double-click entry to jump to line):"
	LISTBOX  7, 18, 194, 108, .SUB_FUNC_ListBox
	PUSHBUTTON  8, 188, 54, 16, .ReloadBn, "&Reload List"
	PUSHBUTTON  76, 188, 60, 16, .BookmarkLineBn, "&Bookmark Line"
	PUSHBUTTON  140, 188, 60, 16, .GotoBookmarkBn, "&Goto Bookmark"
	GROUPBOX  7, 131, 194, 50, .ParametersGroupBox, "Parameters:"
	TEXT  14, 142, 180, 33, .ParametersText, ""
	PROGRESS 7, 213, 194, 11, .Progress
	TEXT  49, 232, 110, 8, .CopyrightText, "Version 1.03   2003  Abe Hendin"
END DIALOG
NavCenterDlg.SUB_FUNC_ListBox.SETSTYLE(8) 'Sort

WITHOBJECT "CorelScript.Automation.8"
	DIM idretval AS INTEGER
	
	CALL LoadPosition
	idretval = DIALOG(NavCenterDlg)
	
SUB NavCenterDlgHandler(BYVAL ControlID%, BYVAL Event%)
	DIM iItem AS INTEGER, sItem AS STRING
	
	SELECT CASE Event%
		'init
		CASE 0
			CALL LoadSubsAndFunctions
			
		'clicking a control
		CASE 2
			IF ControlID% = NavCenterDlg.ReloadBn.GETID() THEN
				CALL LoadSubsAndFunctions
			ENDIF
			
			IF ControlID% = NavCenterDlg.SUB_FUNC_ListBox.GETID() THEN
				iItem = NavCenterDlg.SUB_FUNC_ListBox.GETSELECT()
				sItem = NavCenterDlg.SUB_FUNC_ListBox.GETITEM(iItem)
				FOR i = 1 TO iArrItems
					IF sItem = sEntryArr(i) THEN
						EXIT FOR
					ENDIF
				NEXT
				NavCenterDlg.ParametersText.SETTEXT sParamsArr(i)
			ENDIF
			
			IF ControlID% = NavCenterDlg.BookmarkLineBn.GETID() THEN
				CALL BookmarkLine
			ENDIF
			
			IF ControlID% = NavCenterDlg.GotoBookmarkBn.GETID() THEN
				CALL GotoBookmark
			ENDIF
			
		'Double-clicking a list box item 
		CASE 3
			CurrLineNum = .GetLineNumber() + 1 'series with 0 (first line = 0), apparent bug
			
			'the following lines correct for a bug where .GotoLine fails if text is selected;
			'DO NOT MOVE!
			.AddLineBefore ""
			.GotoLine CurrLineNum
			.DeleteLine
			'END BUG CORRECTION
			
			IF ControlID% = NavCenterDlg.SUB_FUNC_ListBox.GETID() THEN
				iItem = NavCenterDlg.SUB_FUNC_ListBox.GETSELECT()
				sItem = NavCenterDlg.SUB_FUNC_ListBox.GETITEM(iItem)
				FOR i = 1 TO iArrItems
					IF sItem = sEntryArr(i) THEN
						EXIT FOR
					ENDIF
				NEXT
				NavCenterDlg.ParametersText.SETTEXT sParamsArr(i)
				
				.GotoLine sEntryLineNumArr(i)
			ENDIF
	END SELECT
END SUB

SUB LoadPosition
	DIM sini AS STRING, siniDefaults AS STRING, sConfigDir AS STRING
	
	siniDefaults = "0, 0"
	'DlgLeft, DlgTop
	
	sConfigDir = REGISTRYQUERY(2, "SOFTWARE\Corel\Corel Ventura\8.0", "ConfigDir")
	sini = FINDFIRSTFOLDER(sConfigDir + "\SE_NavCenter.ini", 32 OR 128)
	IF sini <> "" THEN 'init file already exists, load settings
		OPEN sConfigDir + "\SE_NavCenter.ini" FOR INPUT AS #1
		INPUT #1, DlgLeft, DlgTop
		CLOSE #1
	ELSE 'init file does not exist: create it with default settings
		OPEN sConfigDir + "\SE_NavCenter.ini" FOR OUTPUT AS #1
		PRINT #1, siniDefaults;
		CLOSE #1
		
		'then load settings
		OPEN sConfigDir + "\SE_NavCenter.ini" FOR INPUT AS #1
		INPUT #1, DlgLeft, DlgTop
		CLOSE #1
	ENDIF
END SUB

SUB SavePosition
	DIM sini AS STRING, sConfigDir AS STRING
	sini = LTRIM(STR(DlgLeft)) + ", " + LTRIM(STR(DlgTop))
	
	'write options to file
	sConfigDir = REGISTRYQUERY(2, "SOFTWARE\Corel\Corel Ventura\8.0", "ConfigDir")
	OPEN sConfigDir + "\SE_NavCenter.ini" FOR OUTPUT AS #1
	PRINT #1, sini;
	CLOSE #1
END SUB

SUB LoadSubsAndFunctions
	DIM sLine AS STRING, sLastLine AS STRING
	DIM iDECLARE AS INTEGER, iSUB AS INTEGER, iFUNCTION AS INTEGER
	DIM iOpenParen AS INTEGER, iCloseParen AS INTEGER
	DIM sItem AS STRING
	DIM bIgnore AS BOOLEAN
	
	'get current dialog window position
	DlgLeft = NavCenterDlg.GETLEFTPOSITION()
	DlgTop = NavCenterDlg.GETTOPPOSITION()
	CALL SavePosition
	
	sLastLine = "0"
	iArrItems = 0
	CurrLineNum = .GetLineNumber() + 1 'series with 0 (first line = 0), apparent bug
	
	'the following lines correct for a bug where .GotoLine fails if text is selected
	.AddLineBefore ""
	.GotoLine CurrLineNum
	.DeleteLine
	'END BUG CORRECTION

	NavCenterDlg.SUB_FUNC_ListBox.RESET 'clear subs and functions list
	NavCenterDlg.ParametersText.SETTEXT "" 'clear params text
	
	.GotoEndOfDoc
	sLastLine = .GetLineNumber() + 1
	IF sLastLine > 0 THEN
		NavCenterDlg.Progress.SETMINRANGE 0
		NavCenterDlg.Progress.SETMAXRANGE INT(sLastLine)
		NavCenterDlg.Progress.SETINCREMENT 1
	ENDIF
	
	FOR i = 1 TO sLastLine
		bIgnore = FALSE
		.GotoLine i
		sLine = .GetLineText()
		iDECLARE = INSTR(sLine, "DECLARE ")
		iSUB = INSTR(sLine, "SUB ")
		iFUNCTION = INSTR(sLine, "FUNCTION ")
		IF INSTR(sLine, "'") = 1 OR INSTR(sLine, "' ") = 1 OR INSTR(sLine, "REM ") = 1 \\
			OR INSTR(sLine, "CALL ") OR iSUB > 5 OR iFUNCTION > 5 \\
			OR INSTR(sLine, "SUB =") > 0 OR INSTR(sLine, "FUNCTION =") > 0 \\
			OR INSTR(sLine, "SUB=") > 0 OR INSTR(sLine, "FUNCTION=") > 0 THEN
			bIgnore = TRUE
		ENDIF
		
		IF iDECLARE = 0 AND (iSUB > 0 OR iFUNCTION > 0) AND NOT bIgnore THEN
			'we have a sub or function; add to list
			iArrItems = iArrItems + 1
			REDIM PRESERVE sEntryArr(iArrItems)
			REDIM PRESERVE sEntryLineNumArr(iArrItems)
			REDIM PRESERVE sParamsArr(iArrItems)
			sEntryLineNumArr(iArrItems) = .GetLineNumber() + 1 'series with 0 (first line = 0), apparent bug
			iOpenParen = INSTR(sLine, "(")
			iCloseParen = INSTR(sLine, ")")
			
			IF iOpenParen > 0 AND iCloseParen > 0 THEN 'params exist
				IF iSUB > 0 THEN
					sEntryArr(iArrItems) = MID(sLine, iSUB, iOpenParen - iSUB)
				ELSEIF iFUNCTION > 0 THEN
					sEntryArr(iArrItems) = MID(sLine, iFUNCTION, iOpenParen - iFUNCTION)
				ENDIF
				sParamsArr(iArrItems) = MID(sLine, iOpenParen + 1, iCloseParen - iOpenParen - 1)
			ELSE 'no params
				IF iSUB > 0 THEN
					sEntryArr(iArrItems) = MID(sLine, iSUB, LEN(sLine) - iSUB + 1)
				ELSEIF iFUNCTION > 0 THEN
					sEntryArr(iArrItems) = MID(sLine, iFUNCTION, LEN(sLine) - iFUNCTION + 1)
				ENDIF
				sParamsArr(iArrItems) = "[None]"
			ENDIF
		ENDIF
		NavCenterDlg.Progress.STEP
	NEXT
		NavCenterDlg.Progress.SETVALUE 0
	
	'set initial selection
	IF iArrItems > 0 THEN
		NavCenterDlg.SUB_FUNC_ListBox.SETARRAY sEntryArr
		NavCenterDlg.SUB_FUNC_ListBox.SETSELECT (1)
		sItem = NavCenterDlg.SUB_FUNC_ListBox.GETITEM(1)
		FOR i = 1 TO iArrItems
			IF sItem = sEntryArr(i) THEN
				EXIT FOR
			ENDIF
		NEXT
		NavCenterDlg.ParametersText.SETTEXT sParamsArr(i)
	ELSE
		NavCenterDlg.SUB_FUNC_ListBox.ADDITEM \\
			"[No subroutines or functions found]"
	ENDIF
	.GotoLine CurrLineNum
END SUB

SUB BookmarkLine
	CurrLineNum = .GetLineNumber() + 1 'starts with 0 (first line = 0), apparent bug
	sFilename = ""
	
	CALL ExtractFilename

	IF LEN(sLoseREM) > 4 THEN
		sFilename = LEFT(sLoseREM, LEN(sLoseREM) - 4) 'drop extension to get filename
		CALL CreateIniFile
		.GotoLine CurrLineNum 'return to starting point
	ELSEIF LEN(sLoseREM) <= 4 THEN
		iretval = MESSAGEBOX("Cannot find bookmark filename. Would " + \\
			CHR(13) + "you like to create a new entry? ", \\
			"Cannot find bookmark filename", MB_EXCLAMATION_ICON + MB_YES_NO)					
		IF iretval = 6 THEN 'yes
			CALL CreateNewFilenameLine
			IF NOT bCancel THEN
				sFilename = LEFT(sLoseREM, LEN(sLoseREM) - 4) 'drop extension to get filename
				CALL CreateIniFile
				.GotoLine CurrLineNum 'return to starting point
			ELSE
				.GotoLine CurrLineNum 'return to starting point
				MESSAGE "Bookmark not created. "
			ENDIF
		ELSE
			.GotoLine CurrLineNum 'return to starting point
			MESSAGE "Bookmark not created. "
		ENDIF
	ENDIF
END SUB

SUB CreateIniFile
	DIM sConfigDir AS STRING
	sConfigDir = REGISTRYQUERY(2, "SOFTWARE\Corel\Corel Ventura\8.0", "ConfigDir")
	SETCURRFOLDER sConfigDir
	OPEN sConfigDir + "\" + sFilename + ".bini" FOR OUTPUT AS #1
	PRINT #1, CurrLineNum
	CLOSE #1
END SUB

SUB GotoBookmark
	DIM sConfigDir AS STRING, sBookmarkLine AS STRING
	
	CurrLineNum = .GetLineNumber() + 1 'starts with 0 (first line = 0), apparent bug
	
	'the following lines correct for a bug where .GotoLine fails if text is selected;
	'DO NOT MOVE! Another bug leads to a content line being deleted if it follows 
	'reading the bini file where the bookmarked line number is stored.
	.AddLineBefore ""
	.GotoLine CurrLineNum
	.DeleteLine
	'END BUG CORRECTION
			
	sFilename = ""
	
	CALL ExtractFilename
	
	IF LEN(sLoseREM) > 4 THEN
		sFilename = LEFT(sLoseREM, LEN(sLoseREM) - 4) 'drop extension to get filename
	
		'read ini file
		sConfigDir = REGISTRYQUERY(2, "SOFTWARE\Corel\Corel Ventura\8.0", "ConfigDir")
		SETCURRFOLDER sConfigDir
		ON ERROR RESUME NEXT
		OPEN sConfigDir + "\" + sFilename + ".bini" FOR INPUT AS #1
		IF ERRNUM = 201 THEN
			.GotoLine CurrLineNum 'return to starting point
			iretval = MESSAGEBOX("Bookmark file does not exist. Please click " + \\
				CHR(13) + "Bookmark Line to recreate.", "Missing bookmark file", \\
				MB_EXCLAMATION_ICON + MB_OK_ONLY)
		ELSE
			LINE INPUT #1, sBookmarkLine
			CLOSE #1
			.GotoLine sBookmarkLine
		ENDIF
	ELSEIF LEN(sLoseREM) <= 4 THEN
		.GotoLine CurrLineNum 'return to starting point
		iretval = MESSAGEBOX("Cannot find bookmark filename. " + \\
			CHR(13) + "Please click Bookmark Line to reset. ", \\
			"Cannot find bookmark filename", MB_EXCLAMATION_ICON + MB_OK_ONLY)
	ENDIF
END SUB

SUB ExtractFilename
	DIM sLine AS STRING
	sLoseREM = ""
	i = 1
	.GotoLine i
	DO
		sLine = RTRIM(.GetLineText())
		IF RIGHT(sLine, 4) = ".csc" THEN
			IF LEFT(sLine, 2) = "' " THEN
				sLoseREM = LTRIM(RIGHT(sLine, LEN(sLine) - 2))
				EXIT DO
			ELSEIF LEFT(sLine, 1) = "'" THEN
				sLoseREM = LTRIM(RIGHT(sLine, LEN(sLine) - 1))
				EXIT DO
			ELSEIF LEFT(sLine, 4) = "REM " THEN
				sLoseREM = LTRIM(RIGHT(sLine, LEN(sLine) - 4))
				EXIT DO
			ELSE
				EXIT DO
			ENDIF
		ELSE
			i = i + 1
			.GotoLine i
		ENDIF
	LOOP UNTIL i > 5
END SUB

SUB CreateNewFilenameLine
	DIM sLineNum AS STRING, bDone AS BOOLEAN, sLineToAdd AS STRING
	
	bDone = FALSE
	bCancel = FALSE
	WHILE NOT bDone AND NOT bCancel 'get filename
		sFilename = INPUTBOX("Enter the filename. Do NOT append " + \\
			CHR(34) + ".csc" + CHR(34) + ".")
		IF sFilename = "" THEN
			iretval = MESSAGEBOX("Do you want to cancel?", \\
				"Invalid filename", \\
				MB_EXCLAMATION_ICON + MB_YES_NO)
			IF iretval = 6 THEN
				.GotoLine CurrLineNum 'return to starting point
				bCancel = TRUE
			ENDIF
		ELSE
			bDone = TRUE
		ENDIF
	WEND
	
	sLineNum = "0"
	bDone = FALSE
	WHILE NOT bDone AND NOT bCancel 'get line number
		sLineNum = INPUTBOX("Enter the line number at which the " + \\
			"filename should be entered (1-5):")
		IF sLineNum = "" THEN sLineNum = "0"
		IF INT(sLineNum) < 1 OR INT(sLineNum) > 5 THEN
			iretval = MESSAGEBOX("Invalid line number. Do you want to cancel?", \\
				"Invalid line number", \\
				MB_EXCLAMATION_ICON + MB_YES_NO)
			IF iretval = 6 THEN
				.GotoLine CurrLineNum 'return to starting point
				bCancel = TRUE
			ENDIF
		ELSE
			bDone = TRUE
		ENDIF
	WEND
	
	IF NOT bCancel THEN
		sLineToAdd = "'" + LTRIM(RTRIM(sFilename)) + ".csc"
		.GotoLine sLineNum
		.AddLineBefore(sLineToAdd)
		CurrLineNum = CurrLineNum + 1
		sLoseREM = RIGHT(sLineToAdd, LEN(sLineToAdd) - 1)
	ENDIF
END SUB

END WITHOBJECT

